project = "GooseSLURM"
copyright = "2018-2022, Tom de Geus"
author = "Tom de Geus"

extensions = [
    "sphinx.ext.autodoc",
    "sphinx.ext.autosummary",
    "sphinx.ext.mathjax",
    "sphinxarg.ext",
]

html_theme = "furo"
exclude_patterns = ["_build", "Thumbs.db", ".DS_Store"]
